package com.nttqonoq.mirza.recordlibrary;

import android.annotation.SuppressLint;
import android.media.AudioDeviceInfo;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class AudioRecorderHandler {

    private static final String TAG = "AudioRecorderHandler";

    private AudioRecord audioRecord;
    private boolean isRecording = false;
    private File outputFile;

    @SuppressLint("MissingPermission")
    public void startRecordingWithDevice(AudioDeviceInfo device, String filePath) {
        if (device == null) {
            Log.e(TAG, "AudioDeviceInfo is null");
            return;
        }

        try {
            outputFile = new File(filePath);
            if (!outputFile.exists()) {
                File parentDir = outputFile.getParentFile();
                if (parentDir != null && !parentDir.exists()) {
                    if (!parentDir.mkdirs()) {
                        Log.e(TAG, "Failed to create directory: " + parentDir.getAbsolutePath());
                        return;
                    }
                }
                if (!outputFile.createNewFile()) {
                    Log.e(TAG, "Failed to create output file: " + filePath);
                    return;
                }
            }

            int sampleRate = 44100; // サンプルレート
            int channelConfig = AudioFormat.CHANNEL_IN_MONO; // モノラル録音
            int audioFormat = AudioFormat.ENCODING_PCM_16BIT; // PCM 16ビット

            // 適切なバッファサイズを計算
            int minBufferSize = AudioRecord.getMinBufferSize(sampleRate, channelConfig, audioFormat);
            if (minBufferSize == AudioRecord.ERROR || minBufferSize == AudioRecord.ERROR_BAD_VALUE) {
                Log.e(TAG, "Invalid buffer size");
                return;
            }

            AudioRecord.Builder builder = new AudioRecord.Builder();
            builder.setAudioSource(MediaRecorder.AudioSource.MIC);
            builder.setAudioFormat(new AudioFormat.Builder()
                    .setEncoding(audioFormat)
                    .setSampleRate(sampleRate)
                    .setChannelMask(channelConfig)
                    .build());
            builder.setBufferSizeInBytes(minBufferSize);

            // AudioRecordを初期化
            audioRecord = builder.build();

            // 録音デバイスを設定
            if (!audioRecord.setPreferredDevice(device)) {
                Log.e(TAG, "Failed to set preferred device");
                return;
            }

            audioRecord.startRecording();
            isRecording = true;

            // 録音データを処理するスレッドを開始
            new Thread(() -> writeAudioDataToFile(minBufferSize)).start();

            // 10秒後に録音を停止する
            // new Handler(Looper.getMainLooper()).postDelayed(this::stopRecording, 10 * 1000);

        } catch (IOException e) {
            Log.e(TAG, "Error initializing recording", e);
        } catch (Exception e) {
            Log.e(TAG, "Unexpected error", e);
        }
    }

    private void writeAudioDataToFile(int bufferSize) {
        try (FileOutputStream fos = new FileOutputStream(outputFile)) {
            byte[] buffer = new byte[bufferSize];
            while (isRecording && audioRecord.getRecordingState() == AudioRecord.RECORDSTATE_RECORDING) {
                int readBytes = audioRecord.read(buffer, 0, buffer.length);
                if (readBytes > 0) {
                    fos.write(buffer, 0, readBytes);
                }
            }
        } catch (IOException e) {
            Log.e(TAG, "Error writing audio data to file", e);
        }
    }

    public void stopRecording() {
        isRecording = false;
        if (audioRecord != null) {
            try {
                audioRecord.stop();
            } catch (IllegalStateException e) {
                Log.e(TAG, "Error stopping recording", e);
            }
            audioRecord.release();
            audioRecord = null;
        }
        Log.i(TAG, "Recording stopped. File saved at: " + getOutputFilePath());
    }

    public String getOutputFilePath() {
        return outputFile != null ? outputFile.getAbsolutePath() : null;
    }
}
